Class: ROM::Changeset::Delete
- Inherits:
-
ROM::Changeset
- Object
- ROM::Changeset
- ROM::Changeset::Delete
- Defined in:
- lib/rom/changeset/delete.rb
Overview
Changeset specialization for delete commands
Delete changesets will execute delete command for its relation, which means proper restricted relations should be used with this changeset.
Instance Attribute Summary
Attributes inherited from ROM::Changeset
#command_options, #command_plugins, #command_type, #relation
Method Summary
Methods inherited from ROM::Changeset
[], #command, #command_compiler_options, command_options, command_plugins, command_type, #commit, #inspect, #new, relation, use